package de.howaner.Pokemon.command;

import de.howaner.Pokemon.PokemonServer;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class CommandManager {
	private final Map<String, Command> commands = new HashMap<String, Command>();
	private final List<String> queuedCommands = Collections.synchronizedList(new ArrayList<String>());

	public CommandManager() {
		// Add default commands
		this.addCommand("stop", new CommandStop());
	}

	public void queueCommand(String command) {
		this.queuedCommands.add(command);
	}

	public void executeQueuedCommands() {
		while (!this.queuedCommands.isEmpty()) {
			String command = this.queuedCommands.get(0);
			this.queuedCommands.remove(0);
			this.executeCommand(command);
		}
	}

	public void executeCommand(String command) {
		Command cmd = this.getCommand(command);
		if (cmd == null) {
			// Do something
			PokemonServer.getServer().getLogger().info("Command not found!");
			return;
		}

		String[] split = command.split(" ");
		String[] args = new String[split.length - 1];

		for (int i = 1; i < split.length; i++) {
			args[i - 1] = split[1];
		}

		cmd.execute(args);
	}

	public Command getCommand(String name) {
		return this.commands.get(name.toLowerCase());
	}

	public void addCommand(String name, Command command) {
		this.commands.put(name.toLowerCase(), command);
	}

	public void removeCommand(String name) {
		this.commands.remove(name.toLowerCase());
	}

	public Set<String> getCommands() {
		return commands.keySet();
	}

}
