package de.howaner.Pokemon.config;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;

public class IniParser {
	private final File file;
	private final Map<String, IniSection> sections = new TreeMap<String, IniSection>();
	private IniSection currentSection;

	public IniParser(File file) {
		this.file = file;
	}

	public IniSection getSection(String name) {
		return this.sections.get(name);
	}

	public IniSection createSection(String name) {
		IniSection section = new IniSection(name);
		this.sections.put(name, section);
		return section;
	}

	public void read() throws Exception {
		this.sections.clear();

		BufferedReader reader = new BufferedReader(new FileReader(this.file));
		String line;

		int lineId = 0;
		while ((line = reader.readLine()) != null) {
			this.readLine(line.trim(), lineId);
		}

		reader.close();
	}

	public void write() throws IOException {
		BufferedWriter writer = new BufferedWriter(new FileWriter(this.file));

		for (IniSection section : this.sections.values()) {
			writer.write("[" + section.getName() + "]");
			writer.newLine();

			for (Entry<String, Object> e : section.getValues().entrySet()) {
				Object value = e.getValue();
				if (value instanceof String[]) {
					value = this.listToString((String[]) value);
				}

				String line = String.format("%s=%s", e.getKey(), value);
				writer.write(line);
				writer.newLine();
			}
		}

		writer.flush();
		writer.close();
	}

	private void readLine(String line, int lineId) throws Exception {
		if (line.contains(";")) {
			int pos = line.indexOf(";");
			line = line.substring(0, pos).trim();
		}
		if (line.isEmpty()) return;

		if (this.isSectionLine(line)) {
			String sectionName = line.substring(1, line.length() - 1);
			this.currentSection = this.getSection(sectionName);
			if (this.currentSection == null) this.currentSection = this.createSection(sectionName);
		}

		if (this.currentSection == null) {
			throw new Exception("Line " + lineId + ": No section");
		}

		String[] split = line.split("=");
		if (split.length != 2) {
			return;
		}

		String key = split[0];
		String value = split[1];
		this.currentSection.set(key, value);
	}

	private boolean isSectionLine(String line) {
		return (line.startsWith("[") && line.endsWith("]"));
	}

	private String listToString(String[] list) {
		StringBuilder builder = new StringBuilder();
		builder.append("[");

		for (int i = 0; i < list.length; i++) {
			if (i != 0) builder.append(",");
			builder.append(list[i]);
		}

		builder.append("]");
		return builder.toString();
	}

}
