package de.howaner.Pokemon.config;

import java.util.Map;
import java.util.TreeMap;
import java.util.regex.Pattern;

public class IniSection {
	private final String name;
	private final Map<String, Object> values = new TreeMap<String, Object>();

	public IniSection(String name) {
		this.name = name;
	}

	public String getName() {
		return this.name;
	}

	public Map<String, Object> getValues() {
		return this.values;
	}

	public void setIfNotAvailable(String key, Object value) {
		if (!this.contains(key) || this.values.get(key).getClass() != value.getClass()) {
			this.values.put(key, value);
		}
	}

	public boolean contains(String key) {
		return this.values.containsKey(key);
	}

	public Object get(String key) {
		return this.values.get(key);
	}

	public int getInteger(String key) {
		return (Integer) this.values.get(key);
	}

	public boolean getBoolean(String key) {
		return (Boolean) this.values.get(key);
	}

	public double getDouble(String key) {
		return (Double) this.values.get(key);
	}

	public String[] getList(String key) {
		return (String[]) this.values.get(key);
	}

	public String getString(String key) {
		return (String) this.values.get(key);
	}

	public void set(String key, Object value) {
		String s = value.toString();

		if (this.isBoolean(s)) {
			this.values.put(key, this.toBoolean(s));
		} else if (this.isInteger(s)) {
			this.values.put(key, this.toInt(s));
		} else if (this.isDouble(s)) {
			this.values.put(key, this.toDouble(s));
		} else if (this.isList(s)) {
			this.values.put(key, this.toList(s));
		} else {
			this.values.put(key, s);
		}
	}

	/////----  Value formats ----/////

	private boolean toBoolean(String value) {
		return value.equalsIgnoreCase("true");
	}

	private int toInt(String value) {
		try {
			return Integer.parseInt(value);
		} catch (NumberFormatException e) {
			return 0;
		}
	}

	private double toDouble(String value) {
		try {
			return Double.parseDouble(value);
		} catch (NumberFormatException e) {
			return 0.0;
		}
	}

	private String[] toList(String value) {
		value = value.substring(1, value.length() - 1);
		return value.split(",");
	}

	private boolean isInteger(String value) {
		return Pattern.matches("[0-9]+", value);
	}

	private boolean isDouble(String value) {
		return Pattern.matches("[0-9]+[.]?[0-9]*", value);
	}

	private boolean isBoolean(String value) {
		return (value.equalsIgnoreCase("true") || value.equalsIgnoreCase("false"));
	}

	private boolean isList(String value) {
		return (value.startsWith("[") && value.endsWith("]"));
	}

}
