package de.howaner.Pokemon.entity;

import de.howaner.Pokemon.PokemonServer;
import de.howaner.Pokemon.listener.event.entity.EntityStartWalkEvent;
import de.howaner.Pokemon.listener.event.entity.EntityWalkedEvent;
import de.howaner.Pokemon.network.packets.out.PacketOutEntityState;
import de.howaner.Pokemon.network.packets.out.PacketOutEntityTeleport;
import de.howaner.Pokemon.network.packets.out.PacketOutEntityWalk;
import de.howaner.Pokemon.network.packets.out.PacketOutFadeEntity;
import de.howaner.Pokemon.util.Location;
import de.howaner.Pokemon.world.World;
import de.howaner.Pokemon.world.WorldLoader.WorldLoadedCallback;

public class Entity {
	private int entityID;
	private Location loc;
	private EntityFace face = EntityFace.SOUTH;
	private EntityState state = EntityState.NORMAL;
	private float moveState = -1f;
	private long lastTickTime;

	public Entity(Location loc) {
		this.loc = loc;
		this.lastTickTime = System.currentTimeMillis();
	}

	public void tick() {
		float deltaTime = (System.currentTimeMillis() - this.lastTickTime) / 1000.0f;
		this.lastTickTime = System.currentTimeMillis();

		this.handleMove(deltaTime);
	}

	public void handleMove(float deltaTime) {
		if (this.moveState == -1f) {
			return;
		}

		if (this.face.getVectorX() == 0) {
			// -> Y coord
			this.moveState += Math.abs(this.face.getVectorY() * 16 * deltaTime * this.getMoveSpeed());
		} else {
			// -> X coord
			this.moveState += Math.abs(this.face.getVectorX() * 16 * deltaTime * this.getMoveSpeed());
		}

		if (this.moveState >= 16f) {
			Location destinition = this.getLocation().clone();
			destinition.add(this.face.getVectorX(), this.face.getVectorY());

			this.moveState = -1f;

			EntityWalkedEvent event = new EntityWalkedEvent(this, destinition);
			PokemonServer.getServer().getEventManager().callEvent(event);
			if (event.isCancelled()) {
				if (this instanceof Player) {
					((Player) this).getClientHandle().clearMovePacketsQueue();
				}
				this.getWorld().broadcastPacket(new PacketOutEntityTeleport(this.entityID, this.loc, this.face), null);
				return;
			}

			this.loc.add(this.face.getVectorX(), this.face.getVectorY());
		}
	}

	public float getMoveSpeed() {
		switch (this.state) {
			case SPRINT: return 5.0f;
			case NORMAL: return 3.5f;
		}
		return 3.5f;
	}

	public boolean isInMove() {
		return (this.moveState != -1f);
	}

	public void cancelMove() {
		this.moveState = -1f;
	}

	public void doMove(EntityFace face) {
		if (this.isInMove()) {
			return;
		}

		this.face = face;

		Location destinition = this.loc.clone();
		destinition.add(this.face.getVectorX(), this.face.getVectorY());

		EntityStartWalkEvent event = new EntityStartWalkEvent(this, destinition);
		PokemonServer.getServer().getEventManager().callEvent(event);
		if (event.isCancelled()) {
			if (this instanceof Player) {
				Player player = (Player) this;
				player.getClientHandle().clearMovePacketsQueue();
				player.getClientHandle().sendPacket(new PacketOutEntityTeleport(player.getEntityID(), player.getLocation(), player.getFace()));
			}
			return;
		}

		this.moveState = 0f;
		this.getWorld().broadcastPacket(new PacketOutEntityWalk(this), ((this instanceof Player) ? ((Player)this).getClientHandle() : null));
	}

	public int getEntityID() {
		return this.entityID;
	}

	public void setEntityID(int entityID) {
		this.entityID = entityID;
	}

	public EntityFace getFace() {
		return this.face;
	}

	public void setFace(EntityFace face) {
		if (this.face == face) {
			return;
		}

		this.face = face;
		this.getWorld().broadcastPacket(new PacketOutEntityTeleport(this.entityID, this.loc, face), ((this instanceof Player) ? ((Player) this).getClientHandle() : null));
	}

	public EntityState getState() {
		return this.state;
	}

	public void setState(EntityState state) {
		if (this.state == state) {
			return;
		}

		this.state = state;
		this.getWorld().broadcastPacket(new PacketOutEntityState(this.entityID, state), ((this instanceof Player) ? ((Player) this).getClientHandle() : null));
	}

	public void teleport(Location location) {
		this.cancelMove();

		if (!location.getWorld().equals(this.loc.getWorld())) {
			// World switch
			this.switchWorld(location);
			return;
		}

		this.loc = location;
		this.getWorld().broadcastPacket(new PacketOutEntityTeleport(this.entityID, location, this.face), null);
	}

	public void switchWorld(final Location newLoc) {
		PokemonServer.getServer().getWorldManager().getWorldLoader().addWorldToQueue(newLoc.getWorld(), new WorldLoadedCallback() {
			@Override
			public void onLoadedWorld(World world) {
				Entity.this.getWorld().removeEntity(Entity.this.getEntityID());
				Entity.this.getLocation().set(world.getName(), newLoc.getPosX(), newLoc.getPosY());
				world.spawnEntity(Entity.this);
			}
		});
	}

	public Location getLocation() {
		return this.loc;
	}

	public World getWorld() {
		return PokemonServer.getServer().getWorldManager().getWorld(this.loc.getWorld());
	}

	public void fadeOut(float duration) {
		this.getWorld().broadcastPacket(new PacketOutFadeEntity(this.entityID, PacketOutFadeEntity.FadeType.FADE_OUT, duration), null);
	}

	public void fadeIn(float duration) {
		this.getWorld().broadcastPacket(new PacketOutFadeEntity(this.entityID, PacketOutFadeEntity.FadeType.FADE_IN, duration), null);
	}

}
