package de.howaner.Pokemon.listener;

import de.howaner.Pokemon.PokemonServer;
import de.howaner.Pokemon.listener.event.Event;
import java.lang.reflect.Method;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

public class EventManager {
	private List<Listener> listeners = new ArrayList<Listener>();

	public void addListener(Listener listener) {
		this.listeners.add(listener);
	}

	public void removeListener(Listener listener) {
		this.listeners.remove(listener);
	}

	public void clearListeners() {
		this.listeners.clear();
	}

	public void callEvent(Event event) {
		List<PresentEvent> foundMethods = new ArrayList<PresentEvent>();

		for (Listener listener : listeners) {
			Method[] methods = listener.getClass().getDeclaredMethods();
			for (Method method : methods) {
				Type[] parameters = method.getGenericParameterTypes();
				if (method.isAnnotationPresent(EventHandler.class) && parameters.length == 1 && parameters[0] == event.getClass()) {
					EventPriority priority = method.getAnnotation(EventHandler.class).priority();
					foundMethods.add(new PresentEvent(method, listener, priority));
				}
			}
		}

		Collections.sort(foundMethods, new Comparator<PresentEvent>() {
			@Override
			public int compare(PresentEvent t, PresentEvent t1) {
				return (t.priority.ordinal() - t1.priority.ordinal());
			}
		});

		for (PresentEvent p : foundMethods) {
			try {
				p.method.setAccessible(true);
				p.method.invoke(p.listener, event);
			} catch (Exception ex) {
				PokemonServer.getServer().getLogger().error("Can't invoke event" + p.method.getName() + " in listener " + p.listener.getClass().getName(), ex);
			}
		}
	}

	public class PresentEvent {
		public Method method;
		public Listener listener;
		public EventPriority priority;

		public PresentEvent(Method method, Listener listener, EventPriority priority) {
			this.method = method;
			this.listener = listener;
			this.priority = priority;
		}
	}

}
