package de.howaner.Pokemon.map.events;

import de.howaner.Pokemon.map.MapTerrain;

public enum EventType {
	ENTITY, WARP, SCRIPT, INTERACT;

	public static EventType getFromId(int id) {
		if (id < 0 || id >= values().length) {
			return null;
		}
		return values()[id];
	}

	public int getId() {
		return this.ordinal();
	}

	public String getName() {
		switch (this) {
			case ENTITY:   return "Entity";
			case WARP:     return "Warp";
			case SCRIPT:   return "Skript";
			case INTERACT: return "Interact";
		}
		return "Unbekannt";
	}

	public String getDescription() {
		switch (this) {
			case ENTITY:   return "Ein Entity, das auf dieser Position steht (z.B. ein NPC)";
			case WARP:     return "Sobald man zu dieser Position geht, wird man zu den angegebenen Coordinaten teleportiert";
			case SCRIPT:   return "Wenn man auf dem Tile steht, wird ein Script ausgeführt.";
			case INTERACT: return "Wenn man dieses Tile anspricht, wird ein Script ausgeführt. (Benötigt BLOCKED Gehdata!)";
		}
		return "Für diesen Type existiert keine Description!";
	}

	public Event createEventInstance(MapTerrain map, int posX, int posY) {
		switch (this) {
			case ENTITY:   return new EventEntity(map, posX, posY);
			case WARP:     return new EventWarp(map, posX, posY);
			case SCRIPT:   return new EventScript(map, posX, posY);
			case INTERACT: return new EventInteract(map, posX, posY);
		}
		return null;
	}

}
