package de.howaner.Pokemon.network.netty;

import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.MessageToByteEncoder;
import javax.crypto.Cipher;

public class NettyEncryptingEncoder extends MessageToByteEncoder {
	private final Cipher cipher;
	private byte[] readCache = new byte[0];
	private byte[] writeCache = new byte[0];

	public NettyEncryptingEncoder(Cipher cipher) {
		this.cipher = cipher;
	}

	private byte[] readBytes(ByteBuf in) {
		int length = in.readableBytes();
		if (this.readCache.length < length) {
			this.readCache = new byte[length];
		}

		in.readBytes(this.readCache, 0, length);
		return this.readCache;
	}

	@Override
	protected void encode(ChannelHandlerContext ctx, Object msg, ByteBuf out) throws Exception {
		if (!(msg instanceof ByteBuf)) {
			return;
		}
		ByteBuf in = (ByteBuf) msg;

		int length = in.readableBytes();
		byte[] bytes = this.readBytes(in);

		int outputSize = this.cipher.getOutputSize(length);
		if (this.writeCache.length < outputSize) {
			this.writeCache = new byte[outputSize];
		}

		out.writeBytes(this.writeCache, 0, this.cipher.update(bytes, 0, length, this.writeCache));
	}

}
