package de.howaner.Pokemon.util;

import de.howaner.Pokemon.PokemonServer;

public class Location {
	private String world;
	private int posX;
	private int posY;

	public Location(String world, int posX, int posY) {
		this.world = world;
		this.posX = posX;
		this.posY = posY;
	}

	public String getWorld() {
		return this.world;
	}

	public int getPosX() {
		return this.posX;
	}

	public int getPosY() {
		return this.posY;
	}

	public void setWorld(String world) {
		this.world = world;
	}

	public void setPosX(int x) {
		this.posX = x;
	}

	public void setPosY(int y) {
		this.posY = y;
	}

	public void set(int x, int y) {
		this.posX = x;
		this.posY = y;
	}

	public void set(String world, int x, int y) {
		this.world = world;
		this.posX = x;
		this.posY = y;
	}

	public void add(Location loc) {
		this.posX += loc.getPosX();
		this.posY += loc.getPosY();
	}

	public void add(int x, int y) {
		this.posX += x;
		this.posY += y;
	}

	public void subtract(int x, int y) {
		this.posX -= x;
		this.posY -= y;
	}

	public void multiply(int factor) {
		this.posX *= factor;
		this.posY *= factor;
	}

	public int distance(Location loc2) {
		return (int) Math.sqrt((this.posX - loc2.posX) ^ 2 + (this.posY - loc2.posY) ^ 2);
	}

	@Override
	public boolean equals(Object obj) {
		if (!(obj instanceof Location)) {
			return false;
		}

		Location loc2 = (Location) obj;
		return (
			this.world.equals(loc2.world) &&
			this.posX == loc2.posX &&
			this.posY == loc2.posY
		);
	}

	public boolean equalsCoords(Location loc2) {
		return (this.posX == loc2.getPosX() && this.posY == loc2.getPosY());
	}

	@Override
	public Location clone() {
		return new Location(this.world, this.posX, this.posY);
	}

	public String toString() {
		return String.format("%s;%d;%d", this.world, this.posX, this.posY);
	}

	public static Location fromString(String s) {
		String[] split = s.split(";");
		if (split.length != 3) {
			return null;
		}

		Location loc = new Location(split[0], 0, 0);
		try {
			loc.set(Integer.parseInt(split[1]), Integer.parseInt(split[2]));
		} catch (Exception ex) {
			return null;
		}
		return loc;
	}

	public static String toJson(Location loc) {
		if (loc == null) return "";
		return PokemonServer.getServer().getGson().toJson(loc);
	}

	public static Location fromJson(String json) {
		if (json == null || json.isEmpty()) return null;
		return PokemonServer.getServer().getGson().fromJson(json, Location.class);
	}

}
