package de.howaner.Pokemon.world;

import de.howaner.Pokemon.entity.Entity;
import de.howaner.Pokemon.entity.Player;
import de.howaner.Pokemon.map.MapTerrain;
import de.howaner.Pokemon.map.MapWalkData;
import de.howaner.Pokemon.map.events.Event;
import de.howaner.Pokemon.network.ClientHandle;
import de.howaner.Pokemon.network.packets.out.OutPacket;
import de.howaner.Pokemon.network.packets.out.PacketOutEntityRemove;
import de.howaner.Pokemon.network.packets.out.PacketOutPlayerSpawn;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class World {
	private final MapTerrain terrain;
	private final Map<Integer, Entity> entities = new HashMap<Integer, Entity>();
	private final List<Player> players = new ArrayList<Player>();

	public void tick() {
		// Tick Entities
		for (Entity entity : entities.values()) {
			entity.tick();
		}
	}

	public int getFreeEntityID() {
		int id = 0;
		while (this.entities.containsKey(id)) {
			id++;
		}

		return id;
	}

	public void spawnEntity(Entity entity) {
		entity.setEntityID(this.getFreeEntityID());
		this.entities.put(entity.getEntityID(), entity);

		if (entity instanceof Player) {
			Player player = (Player) entity;
			this.players.add(player);
			this.broadcastPacket(new PacketOutPlayerSpawn(player), player.getClientHandle());
		}
	}

	public Entity getEntity(int entityID) {
		return this.entities.get(entityID);
	}

	public void removeEntity(int entityID) {
		Entity entity = this.entities.get(entityID);
		if (entity != null) {
			this.entities.remove(entityID);

			if (entity instanceof Player) {
				this.players.remove((Player) entity);
				this.broadcastPacket(new PacketOutEntityRemove(entity), ((Player) entity).getClientHandle());
			} else {
				this.broadcastPacket(new PacketOutEntityRemove(entity), null);
			}
		}
	}

	public Collection<Entity> getEntities() {
		return this.entities.values();
	}

	public World(MapTerrain terrain) {
		this.terrain = terrain;
	}

	public MapTerrain getTerrain() {
		return this.terrain;
	}

	public String getName() {
		return this.terrain.getName();
	}

	public String getJoinMessage() {
		return this.terrain.getJoinMessage();
	}

	public int getWidth() {
		return this.terrain.getWidth();
	}

	public int getHeight() {
		return this.terrain.getHeight();
	}

	public short getTileAt(int x, int y, int layer) {
		return this.terrain.getTileAt(x, y, layer);
	}

	public MapWalkData.WalkDataType getWalkdataAt(int x, int y) {
		return this.terrain.getWalkdataAt(x, y);
	}

	public Event getEventAt(int x, int y) {
		return this.terrain.getEventAt(x, y);
	}

	public List<Player> getPlayers() {
		return this.players;
	}

	public void broadcastPacket(OutPacket packet, ClientHandle ignore) {
		for (Player player : this.getPlayers()) {
			if (player.getClientHandle() == ignore) {
				continue;
			}

			player.getClientHandle().sendPacket(packet);
		}
	}

}
