package de.howaner.Pokemon.world;

import de.howaner.Pokemon.PokemonServer;
import de.howaner.Pokemon.map.MapTerrainMainWorld;
import de.howaner.Pokemon.saveformat.TagCompound;
import de.howaner.Pokemon.saveformat.TagManager;
import de.howaner.Pokemon.world.listeners.WorldChangeListener;
import java.io.File;
import java.util.Collection;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

public class WorldManager {
	private final Map<String, World> worlds = new ConcurrentHashMap<String, World>();
	private World mainWorld;
	private final WorldLoader worldLoader;

	public WorldManager() {
		this.worldLoader = new WorldLoader();
		this.worldLoader.start();
	}

	public WorldLoader getWorldLoader() {
		return this.worldLoader;
	}

	public void addLoadedWorld(World world) {
		this.worlds.put(world.getTerrain().getName(), world);
	}

	public World getWorld(String name) {
		return this.worlds.get(name);
	}

	public Collection<World> getWorlds() {
		return this.worlds.values();
	}

	public World getMainWorld() {
		return this.mainWorld;
	}

	public void onTick() {
		this.worldLoader.onTick(this);

		for (World world : this.getWorlds()) {
			world.tick();
		}
	}

	public void loadMainWorld() throws Exception {
		TagCompound compound = TagManager.readFromFile(new File("maps/Hauptwelt/Hauptwelt.pkmap"));

		MapTerrainMainWorld terrain = new MapTerrainMainWorld();
		terrain.loadFromCompound(compound);

		// Load all parts/chunks
		for (int x = 0; x < (terrain.getWidth() / 40); x++) {
			for (int y = 0; y < (terrain.getHeight() / 40); y++) {
				terrain.loadPart(x, y);
			}
		}

		this.mainWorld = new World(terrain);
		this.worlds.put("Hauptwelt", this.mainWorld);
	}

	public void registerStandardListeners() {
		PokemonServer.getServer().getEventManager().addListener(new WorldChangeListener());
	}

}
